from flask import Flask
from flask_sqlalchemy import SQLAlchemy

from flask_currency.model import CurrencyMixin


class Currency(object):
    """Flask-Currency extension class.

    :param app (Flask): Flask instance
    :param db (SQLAlchemy): Flask-SQLAlchemy instance
    :param currency_class: Currency model class. (optional)
    """
    def __init__(
            self,
            app: Flask,
            db: SQLAlchemy,
            currency_class: type(CurrencyMixin) = None
    ):
        self.app = app
        self.db = db
        self.model = currency_class
        self.init_app(app)

    def init_app(self, app: Flask):
        """Initialize extension with Flask app.

        :param app: Flask application instance
        """
        app.extensions["currency"] = self

        if self.model is None:
            class Currency(self.db.Model, CurrencyMixin):
                pass

            self.model = Currency



import typing

from flask import current_app
from werkzeug.local import LocalProxy

from flask_currency.model import CurrencyMixin

curr = LocalProxy(lambda: current_app.extensions["currency"])


class Money:
    def __init__(
            self,
            amount: float,
            currency: typing.Union[str, CurrencyMixin]
    ):
        self._amount = float(amount)
        if isinstance(currency, CurrencyMixin):
            self._currency = currency
        else:
            self._currency = curr.model.query\
                .filter_by(code=currency).first()

        if self._currency is None:
            raise ValueError("No currency found with code {}".format(currency))

    @property
    def amount(self):
        return self._amount

    @amount.setter
    def amount(self, value: float):
        self._amount = float(value)

    @property
    def currency(self):
        return self._currency

    @currency.setter
    def currency(self, value):
        if isinstance(value, CurrencyMixin):
            currency = value
        else:
            currency = curr.model.query\
                .filter_by(code=value).first()

        self.amount = self.value_in(currency)
        self._currency = currency

    def convert_to(self, currency):
        money = Money(self.amount, self.currency)
        money.currency = currency
        return money

    def value_in(self, currency):
        if not isinstance(currency, CurrencyMixin):
            currency = curr.model.query\
                .filter_by(code=currency).first()

        value = self.amount *  self._currency.value / currency.value

        return round(value, currency.decimals)

    @property
    def formatted(self):
        return self.currency.format(self.amount)

    def __repr__(self):
        return self.formatted

    def __str__(self):
        return self.formatted

    def __html__(self):
        return self.formatted



from flask_sqlalchemy import Model
import sqlalchemy as sa


class CurrencyMixin(Model):
    """Declarative mixin class for currency model."""
    id = sa.Column(sa.Integer, primary_key=True)
    name = sa.Column(sa.String(32))
    code = sa.Column(sa.String(3), unique=True, nullable=False)
    sign = sa.Column(sa.String(4), nullable=False)
    value = sa.Column(sa.Float, nullable=False)
    format_str = sa.Column(sa.String(32))

    @classmethod
    def set_default(cls, currency_code: str):
        """Set currency as base currency with value of 1.

        :param currency_code: Code of base currency
        """
        currency = cls.query.filter_by(code=currency_code).first()
        if currency is None:
            raise ValueError(f'Currency not found: {currency_code}')

        value = currency.value
        for c in cls.query.all():
            c.value /= value

    def set_as_default(self):
        """Set currency as base currency with value of 1."""
        type(self).set_default(self.code)

    def format(self, amount: float):
        """Return formatted string.

        :param amount: Amount of currency.
        :return: Formatted string.
        """
        return self.format_str.format(amount)



from setuptools import setup
from os import path

this_directory = path.abspath(path.dirname(__file__))

with open(path.join(this_directory, 'README.md'), encoding='utf-8') as f:
    long_description = f.read()


setup(
    name='FlaskCurrency',
    version='v1.0',
    packages=['flask_currency'],
    url='https://github.com/gerelorant/Flask-Currency',
    license='MIT',
    author='Gere Lóránt',
    author_email='gerelorant@gmail.com',
    description='Currency handling extension for Flask and SQLAlchemy',
    include_package_data=True,
    long_description=long_description,
    long_description_content_type='text/markdown'
)
